"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fetchImgixDominantColor = exports.fetchImgixBase64Image = exports.buildBase64URL = void 0;
var node_fetch_1 = __importDefault(require("node-fetch"));
var cache_1 = require("../common/cache");
var buildBase64URL = function (contentType, base64) {
    return "data:".concat(contentType, ";base64,").concat(base64);
};
exports.buildBase64URL = buildBase64URL;
/**
 * Given an imgix url, return the base64 image data for the image
 */
var fetchBase64ImageAPI = function (url) { return __awaiter(void 0, void 0, void 0, function () {
    var res, buffer, bufferBase64, error_1;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, (0, node_fetch_1.default)(url)];
            case 1:
                res = _a.sent();
                if (!res.ok) {
                    throw new Error('Something went wrong while fetching the base64 placeholder image');
                }
                _a.label = 2;
            case 2:
                _a.trys.push([2, 4, , 5]);
                return [4 /*yield*/, res.buffer()];
            case 3:
                buffer = _a.sent();
                bufferBase64 = buffer.toString('base64');
                return [2 /*return*/, (0, exports.buildBase64URL)(String(res.headers.get('content-type')), bufferBase64)];
            case 4:
                error_1 = _a.sent();
                throw new Error('Something went wrong while building the base64 placeholder image: ' +
                    String(error_1));
            case 5: return [2 /*return*/];
        }
    });
}); };
/**
 * Fetch the base64 image for an imgix url, and cache the request in the Gatsby
 * cache
 */
var fetchImgixBase64Image = function (cache) { return function (url) {
    return (0, cache_1.withCache)("imgix-gatsby-base64-url-".concat(url), cache, function () {
        return fetchBase64ImageAPI(url);
    });
}; };
exports.fetchImgixBase64Image = fetchImgixBase64Image;
/**
 * Given an imgix url, fetch the dominant color
 */
var fetchImgixDominantColorAPI = function (url) { return __awaiter(void 0, void 0, void 0, function () {
    var res, data, error_2;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, (0, node_fetch_1.default)(url)];
            case 1:
                res = _a.sent();
                if (!res.ok) {
                    throw new Error('Something went wrong while fetching the dominant color for the placeholder image');
                }
                _a.label = 2;
            case 2:
                _a.trys.push([2, 4, , 5]);
                return [4 /*yield*/, res.json()];
            case 3:
                data = (_a.sent());
                return [2 /*return*/, data.dominant_colors.vibrant.hex];
            case 4:
                error_2 = _a.sent();
                throw new Error('Something went wrong while decoding the dominant color for the placeholder image: ' +
                    String(error_2));
            case 5: return [2 /*return*/];
        }
    });
}); };
/**
 * Fetch the dominant color for a given imgix url, and cache the request in the
 * Gatsby cache
 */
var fetchImgixDominantColor = function (cache) { return function (buildURL) {
    var url = buildURL({ palette: 'json' });
    return (0, cache_1.withCache)("imgix-gatsby-dominant-color-".concat(url), cache, function () {
        return fetchImgixDominantColorAPI(url);
    });
}; };
exports.fetchImgixDominantColor = fetchImgixDominantColor;
//# sourceMappingURL=fetchBase64Image.js.map