"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.traceJSON = exports.trace = exports.createLogger = exports.log = void 0;
var debug_1 = __importDefault(require("debug"));
var ns = 'imgix:gatsby';
/**
 * Log a message in the @imgix/gatsby namespace
 */
exports.log = (0, debug_1.default)(ns);
/**
 * Create a custom logger to group logs by a namespace.
 * @param module The namespace to use for the logger
 */
var createLogger = function (module) { return (0, debug_1.default)("".concat(ns, ":").concat(module)); };
exports.createLogger = createLogger;
/**
 * Create a tracer (function that can be used to log a message)
 * @param label A prefix to add to the message
 * @param logger Custom debug logger to use if needed
 * @returns A function that takes a message to log
 */
var trace = function (label, customLogger) { return function (v) {
    (customLogger !== null && customLogger !== void 0 ? customLogger : exports.log)("".concat(label ? "".concat(label, ": ") : '').concat(JSON.stringify(v, null, 2)));
    return v;
}; };
exports.trace = trace;
/**
 * Log out a JSON value.
 * @param rawJSON JSON data to lo
 * @param label Prefix to add to log message
 * @param customLogger Custom debug logger to use if created
 */
var traceJSON = function (rawJSON, label, customLogger) {
    var logger = customLogger !== null && customLogger !== void 0 ? customLogger : exports.log;
    var prefix = label ? "".concat(label, ": ") : '';
    var formattedMessage = JSON.stringify(rawJSON, null, 2);
    logger("".concat(prefix).concat(formattedMessage));
};
exports.traceJSON = traceJSON;
//# sourceMappingURL=log.js.map