import { ComposeInputTypeDefinition, ObjectTypeComposerFieldConfigAsObjectDefinition } from 'graphql-compose';
import { IImgixURLBuilder } from '../../common/imgix-js-core-wrapper';
import { ImgixSourceDataResolver } from '../../common/utils';
import { IImgixParams, ImgixUrlArgs } from '../../publicTypes';
interface CreateImgixUrlFieldConfigArgs<TSource> {
    imgixClient: IImgixURLBuilder;
    resolveUrl: ImgixSourceDataResolver<TSource, string>;
    defaultParams?: IImgixParams;
    paramsInputType: ComposeInputTypeDefinition;
}
/**
 * Create the GraphQL field config for the "url" field that will exist on the
 * imgixImage type
 * @param param0
 * @param param0.imgixClient The imgix client to use to build the URL
 * @param param0.resolveUrl The function to resolve the URL from the source data
 * @param param0.defaultParams The default params to use when building the fixed image URL
 * @param param0.paramsInputType The GraphQL type to use for the params input
 * @returns A GraphQL field config for a "url" size image
 */
export declare const createImgixUrlFieldConfig: <TSource, TContext>({ imgixClient, resolveUrl, defaultParams, paramsInputType, }: CreateImgixUrlFieldConfigArgs<TSource>) => ObjectTypeComposerFieldConfigAsObjectDefinition<TSource, TContext, ImgixUrlArgs>;
export declare const createImgixUrlSchemaFieldConfig: <TSource, TContext>(params: Omit<CreateImgixUrlFieldConfigArgs<unknown>, "imgixClient" | "resolveUrl"> & {
    imgixClientOptions?: import("../../common/imgix-js-core-wrapper").CreateImgixClientParams | undefined;
    resolveUrl?: ((source: TSource) => string | null | undefined) | undefined;
}) => ObjectTypeComposerFieldConfigAsObjectDefinition<TSource, TContext, any>;
export {};
//# sourceMappingURL=createImgixUrlFieldConfig.d.ts.map