"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSchemaCustomization = void 0;
var graphql_1 = require("gatsby/graphql");
var lodash_get_1 = __importDefault(require("lodash.get"));
var ramda_1 = require("ramda");
var __1 = require("../..");
var constants_1 = require("../../common/constants");
var imgix_js_core_wrapper_1 = require("../../common/imgix-js-core-wrapper");
var utils_1 = require("../../common/utils");
var typeBuilder_1 = require("./typeBuilder");
function isStringArray(value) {
    return (Array.isArray(value) &&
        value.every(function (element) { return typeof element === 'string'; }));
}
/**
 * Given a GraphQL node (instance of a type), and the plugin options, fetch the
 * image url to proxy through imgix.
 * @param fieldOptions The options object from gatsby-config.js
 * @param node The GraphQL node to search for the image URL in
 * @returns The image URL(s) if found, or throws an error
 */
var getFieldValue = function (_a) {
    var fieldOptions = _a.fieldOptions, node = _a.node;
    var prefixWithURLPrefix = function (url) {
        return (fieldOptions.URLPrefix || '') + url;
    };
    if ('rawURLKey' in fieldOptions) {
        // Here we should be only fetching one url from the node
        var url = (0, lodash_get_1.default)(node, fieldOptions.rawURLKey);
        if (url == null || typeof url !== 'string') {
            throw new Error('rawURLKey must reference a URL string');
        }
        return prefixWithURLPrefix(url);
    }
    else if ('rawURLKeys' in fieldOptions) {
        // Here we are looking to fetch multiple urls
        var urls = fieldOptions.rawURLKeys.map(function (rawURLKey) {
            return (0, lodash_get_1.default)(node, rawURLKey);
        });
        if (!isStringArray(urls)) {
            throw new Error('rawURLKeys must reference a list of URL strings');
        }
        return urls.map(prefixWithURLPrefix);
    }
    var _neverReturn = fieldOptions; // Fixes typescript error 'not all code paths return a value'
    return _neverReturn;
};
/**
 * "Decode" the options that the user set in gatsby-config.js, to verify that
 * they match our expected options schema. If they don't, an error will be
 * thrown.
 * @param options The options object from gatsby-config.js
 * @returns The options object if it is valid, or throws an error
 */
var decodeOptions = function (options) {
    if (options.sourceType === 'webProxy' &&
        (options.secureURLToken == null || options.secureURLToken.trim() === '')) {
        throw new Error("The plugin option 'secureURLToken' is required when sourceType is 'webProxy'.");
    }
    if (options.fields != null && !Array.isArray(options.fields)) {
        throw new Error('Fields must be an array of field options');
    }
    if (options.sourceType === __1.ImgixSourceType.WebProxy &&
        (options.secureURLToken == null || options.secureURLToken.trim() === '')) {
        throw new Error('A secure URL token must be provided if sourceType is webProxy');
    }
    return options;
};
var setupImgixClient = function (_a) {
    var options = _a.options, packageVersion = _a.packageVersion;
    return (0, imgix_js_core_wrapper_1.createImgixURLBuilder)({
        domain: options.domain,
        secureURLToken: options.secureURLToken,
        ixlib: options.disableIxlibParam !== true
            ? "gatsbySourceUrl-".concat(packageVersion)
            : undefined,
    });
};
/**
 * createSchemaCustomization is a Gatsby core API hook which can be used to
 * update the GraphQL schema.
 * Here, we use it to add our imgix GraphQL types, as well as add fields to the
 * corresponding types, which are the types that the user has specified to
 * transform, as well as the the root Query type (this adds the imgixImage type
 * to the root query)
 */
var createSchemaCustomization = function (gatsbyContext, _options) { return __awaiter(void 0, void 0, void 0, function () {
    var validatedOptions, errorString, imgixClient, typesAndFields, imgixImageType, optionsFields, fieldTypes, rootType, createTypes, errorString;
    var _a;
    return __generator(this, function (_b) {
        try {
            validatedOptions = decodeOptions(_options);
        }
        catch (error) {
            errorString = "[@imgix/gatsby] Fatal error during setup, plugin options are not valid: ".concat(String(error));
            gatsbyContext.reporter.panic(errorString);
            throw new Error(errorString);
        }
        imgixClient = setupImgixClient({
            options: validatedOptions,
            packageVersion: constants_1.VERSION,
        });
        typesAndFields = (0, typeBuilder_1.buildImgixGatsbyTypes)({
            cache: gatsbyContext.cache,
            imgixClient: imgixClient,
            resolveUrl: (0, ramda_1.prop)('rawURL'),
            defaultParams: validatedOptions.defaultImgixParams,
        });
        imgixImageType = gatsbyContext.schema.buildObjectType({
            name: 'ImgixImage',
            fields: {
                url: typesAndFields.fields.url,
                fluid: typesAndFields.fields.fluid,
                fixed: typesAndFields.fields.fixed,
                gatsbyImageData: typesAndFields.fields.gatsbyImageData,
            },
        });
        optionsFields = (_a = validatedOptions.fields) !== null && _a !== void 0 ? _a : [];
        fieldTypes = createFieldTypes(optionsFields, gatsbyContext, imgixImageType);
        rootType = createRootObjectType(gatsbyContext, imgixImageType);
        // Now, we actually add the types and fields to the schema
        // For more information refer to the docs at
        // https://www.gatsbyjs.com/docs/reference/config-files/actions/#createTypes
        // and
        // https://www.gatsbyjs.com/docs/reference/graphql-data-layer/schema-customization/
        try {
            createTypes = gatsbyContext.actions.createTypes;
            createTypes(typesAndFields.types.map(gatsbyContext.schema.buildObjectType));
            createTypes(typesAndFields.enumTypes.map(gatsbyContext.schema.buildEnumType));
            createTypes(typesAndFields.inputTypes.map(gatsbyContext.schema.buildInputObjectType));
            createTypes(fieldTypes);
            createTypes(imgixImageType);
            createTypes(rootType);
        }
        catch (error) {
            errorString = '[@imgix/gatsby] Error during setup when creating GraphQL types: ' +
                String(error);
            gatsbyContext.reporter.panic(errorString);
            throw new Error(errorString);
        }
        return [2 /*return*/];
    });
}); };
exports.createSchemaCustomization = createSchemaCustomization;
/**
 * This "creates" a 'Query' type, with our imgixImage type as a field. Gatsby
 * will then later merge this with its own Query type, effectively adding our
 * imgixImage field to the root of the GraphQL schema.
 * @param gatsbyContext Gatsby context object passed into Gatsby hooks
 * @param imgixImageType The imgix image GraphQL type
 * @returns A GraphQL object type representing the GraphQL Query type.
 */
function createRootObjectType(gatsbyContext, imgixImageType) {
    return gatsbyContext.schema.buildObjectType({
        // 'Query' here refers to the root GraphQL Query type
        name: 'Query',
        fields: {
            imgixImage: {
                type: imgixImageType.config.name,
                resolve: function (_, args) {
                    // If url is not defined, return null. In reality this shouldn't
                    // happen due to the GraphQL type definition (below) ensuring that
                    // this is not null.
                    if ((args === null || args === void 0 ? void 0 : args.url) == null || typeof (args === null || args === void 0 ? void 0 : args.url) !== 'string') {
                        return null;
                    }
                    // This "raw url" is transformed later in the resolvers for the fields
                    // that are children of this type
                    return { rawURL: args === null || args === void 0 ? void 0 : args.url };
                },
                args: {
                    url: {
                        type: new graphql_1.GraphQLNonNull(graphql_1.GraphQLString),
                        description: 'The path of the image to render. If using a Web Proxy Source, this must be a fully-qualified URL.',
                    },
                },
            },
        },
    });
}
/**
 * This function "creates" types for the types that the user has specified that
 * we should modify. Gatsby will later merge these with the existing types,
 * adding our imgixImage field to the types that the user has specified.
 * @param optionsFields The fields that the user has specified to modify
 * @param gatsbyContext Gatsby context object passed into Gatsby hooks
 * @param imgixImageType The imgix image GraphQL type
 * @returns An array of GraphQL types, corresponding to the types that the user has specified to modify
 */
function createFieldTypes(optionsFields, gatsbyContext, imgixImageType) {
    return optionsFields.map(function (fieldOptions) {
        var _a;
        return gatsbyContext.schema.buildObjectType({
            name: "".concat(fieldOptions.nodeType),
            // We have to declare that we're extending the Node interface here
            // This does the same as 'implements Node' does for SDL-defined types.
            // See here for more info:
            // https://www.gatsbyjs.com/docs/reference/graphql-data-layer/schema-customization/#:~:text=when%20defining%20top-level%20types%2C%20don%E2%80%99t%20forget%20to%20pass%20interfaces%3A%20%5B'node'%5D%2C%20which%20does%20the%20same%20for%20type%20builders%20as%20adding%20implements%20node%20does%20for%20sdl-defined%20types.%20
            interfaces: ['Node'],
            fields: (_a = {},
                _a[fieldOptions.fieldName] = {
                    type: 'rawURLKeys' in fieldOptions
                        ? "[".concat(imgixImageType.config.name, "]")
                        : imgixImageType.config.name,
                    resolve: function (node) {
                        try {
                            var rawURL = getFieldValue({
                                fieldOptions: fieldOptions,
                                node: node,
                            });
                            return {
                                rawURL: rawURL,
                            };
                        }
                        catch (error) {
                            // If no urls are found, try to find possible image urls in the
                            // data, and if some are found, try to provide a useful error
                            // message
                            var urlPathsFound_1 = typeof node === 'object' && node != null
                                ? (0, utils_1.findPossibleURLsInNode)(node)
                                : [];
                            var potentialImagesString = (function () {
                                if (urlPathsFound_1.length === 0) {
                                    return '';
                                }
                                var output = '';
                                output += 'Potential images were found at these paths:\n';
                                urlPathsFound_1.map(function (_a) {
                                    var path = _a.path, value = _a.value;
                                    output += " - ".concat(path, "\n");
                                    if (value.startsWith('http')) {
                                        output += '   Set following configuration options:\n';
                                        output += "     rawURLKey: '".concat(path, "'\n");
                                        output += "     URLPrefix: 'https:'\n";
                                    }
                                    else {
                                        output += '   Set following configuration option:\n';
                                        output += "     rawURLKey: '".concat(path, "'\n");
                                    }
                                });
                                return output;
                            })();
                            gatsbyContext.reporter.panic("Error when resolving URL value for node type ".concat(fieldOptions.nodeType, ". This probably means that the rawURLKey function in gatsby-config.js is incorrectly set. Please read this project's README for detailed instructions on how to set this correctly.\n                      \n").concat(potentialImagesString));
                        }
                    },
                },
                _a),
        });
    });
}
//# sourceMappingURL=gatsby-node.js.map